##Copyright 2008 Petra Kralj
##Contact: Petra.Kralj@ijs.si
##
##This file is part of GMOtrack.
##
##GMOtrack is free software: you can redistribute it and/or modify
##it under the terms of the GNU General Public License as published
##by the Free Software Foundation, either version 3 of the License,
##or (at your option) any later version.
##
##GMOtrack is distributed in the hope that it will be useful, but
##WITHOUT ANY WARRANTY; without even the implied warranty of
##MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##GNU General Public License for more details.
##
##You should have received a copy of the GNU General Public License
##along with GMOtrack. If not, see <http://www.gnu.org/licenses/>.


from debug import *

import string
import sets
import math
import operator

import orange

true = 1
false = 0

def tobin(x, count=8):
    """
    Integer to binary string
    Count is number of bits
    """
    return "".join(map(lambda y:str((x>>y)&1), range(count-1, -1, -1)))


def entropy (arr):
    """Calculates the entropy of an array of numbers."""
    sumArr = sum(arr) *1.0
    ent = 0
    for a in arr:
        if a>0:
            pi = a/sumArr
            ent = ent - pi*math.log(pi,2)
    return ent
 
def fact(x):
    if x<=1:
        return 1
    else:
        return reduce(lambda x, y: x*y, xrange(2, x+1))

def combinations(n,r):
    """Number of unique combinations of n items of size up to r. """
    combinations = 0
    for i in range (1,min(r+1,n)):
        level_i= fact(n) /( fact (n-i) * fact (i))
        combinations = combinations + level_i
    return combinations

def xuniqueCombinations(items, n):
    if n==0: yield []
    else:
        for i in xrange(len(items)-n+1):
            for cc in xuniqueCombinations(items[i+1:],n-1):
                yield [items[i]]+cc
                
#-----------------------------------------------------------------------------------------------------------



class ComputatationGMOtrack:
    def __init__(self, dataGMO, minCoverage, m, edtStatus=None, k=10, n=100, outputall=1):
        self.dataGMO = dataGMO              # ExampleTableGMO  -- dataGMO.table is an orange.ExampleTable, other added functionality
        self.data = dataGMO.table           # orange.ExampleTable
        self.minCoverage = minCoverage      # in [0..100]
        self.m = m                          # max number of assays per assay set
        self.k = k
        self.n = n
        self.outputAll =outputall
        
        self.edtStatus = edtStatus          # qt.multilineEdit for notifications

        self.assaySets = []
        self.generateassaySets()
        notify ("computation finished",self.edtStatus)
        """
        self.assaySets contains all the results of the computation.
       
        """

    def generateassaySets(self):
        """Generate sets of indexes."""
        notify("generating assay sets candidates (%d combinations to evaluate)"%combinations(self.dataGMO.nAssays, self.m),self.edtStatus)
        rez=[]
        n=len(self.dataGMO.table.domain.variables)
        items = range ( n)
        for l in range (1,self.m+1):
            newLevel = xuniqueCombinations(items,l)
            rez.append(newLevel)
        if self.outputAll ==1:
            self.codeAsFilters(rez)
        else:
            self.codeAsFilterstBest(rez)


    def codeAsFilters(self,rez):
        comb = combinations (self.dataGMO.nAssays, self.m)*1.0
        notify("evaluating ...", self.edtStatus)
        AssaySet.dataGMO = None
        i=0
        for level in rez:
            for set in level:
                i+=1
                if (i % 5000)==0:
                    notify(" (%2.1f%%)"%(i/comb*100), self.edtStatus)
                fAssay = AssaySet(set, self.dataGMO, self.k, self.n)
                if fAssay.coverage >= self.minCoverage:
                    fAssay.evaluate()
                    self.assaySets.append(fAssay)

    def codeAsFilterstBest(self,rez):
        comb = combinations (self.dataGMO.nAssays, self.m)*1.0
        notify("evaluating ...", self.edtStatus)
        AssaySet.dataGMO = None
        i=0
        for level in rez:
            topAssay = AssaySet([0], self.dataGMO, self.k, self.n)
            for set in level:
                fAssay = AssaySet(set, self.dataGMO, self.k, self.n)
                if fAssay.coverage >= self.minCoverage:
                    fAssay.evaluate()
                    if fAssay.cost < topAssay.cost:
                        topAssay = fAssay
                i+=1
                if (i % 4000)==0:
                    notify(" (%2.1f%%) "%(i/comb*100), self.edtStatus)
            if topAssay.cost != 9999999999:
                self.assaySets.append(topAssay)


    def computeCost(self,k,n):
        for assaySet in self.assaySets:
            assaySet.setkn(k,n)
            assaySet.computeCost()


    # debug functions
    def printItemSets(self,rez):
        for level in rez:
            print "level"
            for set in level:
                print set                    


#--------------------------------------------------------------------------------------------------------------------
#--------------------------------------------------------------------------------------------------------------------
#--------------------------------------------------------------------------------------------------------------------
class AssaySet:
    dataGMO = None

    def initializeStatic(self, dataGMO, k, n):
        """ Initializes static variables and dictionaries that are common for all instances of AssaySet"""
        if AssaySet.dataGMO == None:
            AssaySet.dataGMO = dataGMO
            AssaySet.k=k
            AssaySet.n=n
            AssaySet.conditionsMap = []
            for i in range(len(AssaySet.dataGMO.table.domain.variables)):
                ov = orange.ValueFilter_discrete( position = i, values = [orange.Value(dataGMO.table.domain.variables[i], "1")])
                AssaySet.conditionsMap.append(ov)
            AssaySet.q = {}  # a dictionary containing probabilities that all of the assays in the key touple are negative
                    # key is tuple(AssaySet.attributeIndexes)
            
            
        
    def __init__(self, attributeIndexes, dataGMO, k, n):
        """attributes - list of integers rapresenting indexes of attributes in data.domain.variables"""
        #Code attribute indexes as orange.filter with the domain data.domain. 
        self.initializeStatic(dataGMO, k, n)

        self.attributeIndexes = attributeIndexes
        self.nconds = len(attributeIndexes)

        #assaySetName is a concatenation of assay names with spaces in between
        self.assaySetName = reduce(operator.concat, [ (AssaySet.dataGMO.table.domain.variables[i].name + "    ") for i in attributeIndexes ])

        #aconditions in filter are orange.ValueFilter_discrete according to attributeIndexes mapped by AssaySet.conditionsMap
        conds = [ AssaySet.conditionsMap[i] for i in attributeIndexes]
        self.filter = orange.Filter_values( domain = AssaySet.dataGMO.table.domain, conditions = conds,  conjunction = 0 )
        self.selectedData = self.filter(AssaySet.dataGMO.table)
        
        #Coverage and q=(probability that all the assays are negative)
        self.coverage = len(self.selectedData)*100.0/len(AssaySet.dataGMO.table)
        self.qAssaySet=1       # q is the probability that all the assays are negative
        for d in self.selectedData:
            self.qAssaySet = self.qAssaySet * (1 - d.getmeta(AssaySet.dataGMO.freqName))
        AssaySet.q[tuple(self.attributeIndexes)]=self.qAssaySet

        # Evaluation measures based on outcomes
        self.cost = 9999999999
        self.comTxt = []
        self.numAbsentPerComb = []
        self.numPossiblePerComb = []
        self.probabilityPerComb = []
        self.costPerComb = []
        

    def setkn(self,k,n):
        AssaySet.k = k
        AssaySet.n = n


    def evaluate(self):
        """Evaluate AssaySet: for each possible outcome calculate #absent, #possible and probability of outcome."""
        outcomes = 2**self.nconds               # number of possible outcomes = 2^(number of conditions)
        for j in range(outcomes):
            txt= tobin(j, self.nconds)          # code outcome ID as binary string
            condsA = []
            for i in range(self.nconds):
                pos = self.filter.conditions[i].position
                if txt[i]=="0":                 # Absent
                  condsA.append(AssaySet.conditionsMap[self.attributeIndexes[i]])
            fPossible = orange.Filter_values( domain = self.filter.domain, conditions = condsA,  conjunction = 0 )
            fPossible.negate = 1

            possibleGMOdata = fPossible(self.dataGMO.table)

            nPossible = len(possibleGMOdata)
          
            self.comTxt.append(txt)
            self.numPossiblePerComb.append(nPossible)
            self.numAbsentPerComb.append(len(self.dataGMO.table)-nPossible)
            self.computeProbability(j, outcomes, txt)
        self.computeCost()
        self.selectedData = None  # clear memory





    def computeProbability(self, comb, outcomes, txt):
        if comb==0:                                         # case 0000..000
            p = self.qAssaySet
        elif comb==outcomes-1:                              # case 1111..111
            p = 1-sum(self.probabilityPerComb)
        elif string.count(txt,"1")==1:                      # case exactly one '1' all the rest is '0' or only one "1"
            index = txt.find("1")
            key = self.attributeIndexes[0:index]+self.attributeIndexes[index+1:]
            p = AssaySet.q[tuple(key)] - self.qAssaySet     # the formula for the probability = q(U(all zeros)) - q(U(all)))
        else:                                       # here comes the general formula, but the most common cases can be computed faster
            Ipos = []     # indexes of positive tests
            Ineg = []     # indexes of negative tests
          ##  print txt , "+", self.attributeIndexes,"\t-->"
            for i in range(len(txt)):
                if txt[i] =="1" :    Ipos.append(self.attributeIndexes[i])
                else:                Ineg.append(self.attributeIndexes[i])
        ##    print txt,Ineg
            p=AssaySet.q[tuple(Ineg)]
            for i in range(1, len(txt)+1):         # |K|
                sign = (-1)**i
                subsets = xuniqueCombinations(Ipos,i)
                for set in subsets:
                    key = set+Ineg       #!!!!!!!!!!!!!!!!!!!!!!!!
                    key.sort()
                   # print "\t", key,"\t-->", sign,"%1.3f"%AssaySet.q[tuple(key)]
                    p = p + sign * AssaySet.q[tuple(key)]   #!!!!!!!!!!!!!!!!!!!1
                    


        self.probabilityPerComb.append(p)  







    def computeCost(self):
        self.cost = AssaySet.k * self.nconds + AssaySet.n     # cost for screening depends on the number of screening assays
        # cost for event specific assays depends on the number of possible GMOs after screening
        self.costPerComb = []
        for i in range(len(self.comTxt)):                   # for each possible outcome
            if self.numPossiblePerComb[i]!=0:
                costi = AssaySet.k*self.numPossiblePerComb[i]+AssaySet.n       # cost in case of this outcome
            else: costi=0
            self.costPerComb.append(costi)
            self.cost = self.cost + self.probabilityPerComb[i] * costi  # weighted sum, weight is the probability of outcome




if __name__=="__main__":
    filename = "D:\\_Work\\Projects\\CRPs\\CRP-V4-314-sledljivostGMO\\Data\\GMOfrequencies.tab"
    data = orange.ExampleTable(filename)
    assaySets=ComputatationSledljivostGMO(data, 80, 3)
