##Copyright 2008 Petra Kralj
##Contact: Petra.Kralj@ijs.si
##
##This file is part of GMOtrack.
##
##GMOtrack is free software: you can redistribute it and/or modify
##it under the terms of the GNU General Public License as published
##by the Free Software Foundation, either version 3 of the License,
##or (at your option) any later version.
##
##GMOtrack is distributed in the hope that it will be useful, but
##WITHOUT ANY WARRANTY; without even the implied warranty of
##MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##GNU General Public License for more details.
##
##You should have received a copy of the GNU General Public License
##along with GMOtrack. If not, see <http://www.gnu.org/licenses/>.



import orange



class ExampleTableGMO:
    def __init__(self):
        self.table = []
        self.orgTable = []
        self.nAssays = 0
        self.nGMOs = 0
        self.setFreqName(0)
        

    def setFreqName(self, index):
        self.freqName = "probability"

    def adjustNums(self):
        self.nAssays = len(self.table.domain)
        self.nGMOs = len(self.table)

    
    def loadData(self, filename):
        self.orgTable = orange.ExampleTable(filename)
        self.table = self.orgTable
        self.adjustNums()


    def setData(self, data):
        self.table = data
        self.adjustNums()
        

    def selectGMOsBySpecies(self, selSpecies):
        """The data of the selected specis are filtered and stored into self.selectedData ."""
        if self.orgTable != []:
            self.table= orange.ExampleTable(self.orgTable.domain)
            for d in self.orgTable:
                if d.getmeta("species") in selSpecies:
                    self.table.append(d)
            self.removeAbsIrrelevantAssays()
            self.removeInterchangableAssays()
            self.adjustNums()



    def removeAbsIrrelevantAssays(self):
        """Finds and removes absolutely irrelevant assays = assays with all zeros."""
        if self.table!= []:
            rel = []
            # find absolutely relevant variables = all but those with zeros only 
            for i in range(len(self.table.domain.variables)):
                for d in self.table:
                    if d[i]!="0":
                        rel.append(self.table.domain.variables[i])
                        break
            nAabIrrelevant  = len(self.table.domain.variables)-len(rel)
            domain = orange.Domain(rel)
            domain.addmetas(self.table.domain.getmetas()) # meta variables 
            self.table = orange.ExampleTable(domain, self.table)
            self.adjustNums()
        

    def removeInterchangableAssays(self):
        """Checks for assays with identical columns = interchangable assays."""
        if self.table!= [] and len(self.table.domain) > 3:
            #find relatively relevant variables = remove the second of two identical columns
            irr = []     # list of indexes of irrelevant assays
            for i in range(len(self.table.domain.variables)-1):
                for j in range(i+1,len(self.table.domain.variables)):
                    if j not in irr:
                        irrelevant = 1
                        for d in self.table:
                            if d[self.table.domain.variables[i]]!= d[self.table.domain.variables[j]]:                               
                                irrelevant = 0
                                break
                        if irrelevant == 1:
                            irr.append(j)

            rel = []
            # meta variables at the beginning
            for i in range(len(self.table.domain.variables)):
                if i not in irr:
                    rel.append(self.table.domain.variables[i])  
            domain = orange.Domain(rel)
            domain.addmetas(self.table.domain.getmetas()) # meta variables 
            nInterchangableAssays = len(self.table.domain)-len(rel)
            self.table = orange.ExampleTable(domain, self.table)
            self.adjustNums()


    def calculateFrequencies(self):
        if self.table!=[]:
            for a in self.table.domain.attributes:
                a.q = 1
                for d in self.table:
                    if d[a]=="1":
                        a.q = a.q * (1-d.getmeta(self.freqName))
                a.probability=1-a.q


            
    def printBasicInfo(self):
        print "nAssays: " , self.nAssays
        print "nGMOs:  " , self.nGMOs
        print "nmetas: " , len(self.table.domain.getmetas())




if __name__=="__main__":
    filename = "D:\\_Work\\Projects\\CRPs\\CRP-V4-314-sledljivostGMO\\Data\\GMOfrequencies.tab"
    data = ExampleTableGMO()
    data.loadData(filename)
    data.printBasicInfo()
    data.selectGMOsBySpecies(["maize"])
    data.printBasicInfo()
    
    
