##Copyright 2008 Petra Kralj
##Contact: Petra.Kralj@ijs.si
##
##This file is part of GMOtrack.
##
##GMOtrack is free software: you can redistribute it and/or modify
##it under the terms of the GNU General Public License as published
##by the Free Software Foundation, either version 3 of the License,
##or (at your option) any later version.
##
##GMOtrack is distributed in the hope that it will be useful, but
##WITHOUT ANY WARRANTY; without even the implied warranty of
##MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##GNU General Public License for more details.
##
##You should have received a copy of the GNU General Public License
##along with GMOtrack. If not, see <http://www.gnu.org/licenses/>.




from debug import *
import sys
import getopt

import orange

from ExampleTableGMO import *

from ComputatationGMOtrack import *


true = 1
false = 0

"""
This file contains mainly the interface of the GMOtracking program.
A part of the interface is also in the file ShowData.py.
The computation part is in the file ComputatationSledljivostGMO.py.

"""



def printAssaySets(computationRezult):
        """Print assay sets."""
        notify("printing results\n")
        print "\t__________________________________________________________________"
        for assaySet in computationRezult.assaySets:
            printAssaySet(assaySet)
            print "\t__________________________________________________________________"


def printAssaySet(assaySet):
        """Print single assay set."""
        tNum = len(assaySet.filter.conditions)
        print ("\tassays                       \t"+assaySet.assaySetName)
        print ("\tnumber of assays             \t"+str(tNum))
        print ("\tcoverage                     \t%2.0f %%"%assaySet.coverage)
        scrCost = tNum*assaySet.k+assaySet.n
        print ("\tscreening cost               \t%4.2f Eur"%scrCost)
        eesCost = assaySet.cost-scrCost
        print ("\texpected event-specific cost \t%4.2f Eur"%eesCost)
        print ("\texpected total cost          \t%4.2f Eur"%assaySet.cost)
        print ("\tdetails:")
        print     ("\t  outcome \t probability \t #possible \t cost ")
        for i in range(len(assaySet.comTxt)):
            print ("\t  %s     \t %1.3f       \t %s          \t %4.1f"%(
            assaySet.comTxt[i],
            abs(assaySet.probabilityPerComb[i]),
            str(assaySet.numPossiblePerComb[i]),
            assaySet.costPerComb[i]))


def usage():
        notify("\nGMOtrack usage:\n",addTimeStamp=false)
        notify("GMOtrack filename [-m int][-c float][-k float][-n float]\n",addTimeStamp=false)
        notify("\t -m\t maximum assays for first phase constraint, default m=5\n",addTimeStamp=false)
        notify("\t -c\t minimum coverage (in %) of the first phase, default c=80\n",addTimeStamp=false)
        notify("\t -k\t coefficient for the cost function g, default k=21.18\n",addTimeStamp=false)
        notify("\t -n\t constant for the cost function g, default n=91.82\n",addTimeStamp=false)
        notify("The program returns for each number i up to m the combination of \n",addTimeStamp=false)
        notify("i screening assays with the lowest expected total cost.",addTimeStamp=false)
        notify("\n",addTimeStamp=false)


def main(argv):
    m = 3 #5
    c = 80
    k = 21.18
    n = 91.82
    #parse command line parameters - set m,c,k,n
    if len(argv) == 0:
        print "Error: no input data"
        usage()
        sys.exit(3)
    filename = argv[0]
    try:
        opts, args = getopt.getopt(argv[1:], "m:c:n:k:", [])
    except getopt.GetoptError:
        usage()
        sys.exit(2)
    for opt, arg in opts:
        if opt == '-m':
            m=eval(arg)
        elif opt == '-c':
            c=eval(arg)*1.0
        elif opt == '-k':
            k=eval(arg)*1.0
        elif opt == '-n':
            n=eval(arg)*1.0
    notify("\nParameters:\n",addTimeStamp=false)
    notify("\tm = %2d    \t- maximum assays for first phase constraint\n"%m,addTimeStamp=false)
    notify("\tc = %2.0f%% \t- minumum coverage of the first phase\n"%c,addTimeStamp=false)
    notify("\tg = %2.2f * |numAssays| + %2.2f  - linear cost function g \n"%(k,n),addTimeStamp=false)
    notify("\tdata: \t%s\n"%filename,addTimeStamp=false)
    dataGMO = ExampleTableGMO()
    try:  # load data from self.filename
        notify("loading data")
        dataGMO.loadData(filename)
    except Exception:
        notify ("Error when reading data: wrong path, filename or file format\n\n")
        sys.exit(4)
    notify("computation started")
    computationRezult = ComputatationGMOtrack( dataGMO, c, m, None, k, n, 0)
    printAssaySets(computationRezult)



if __name__ == "__main__":
    if len(sys.argv)==1:
        usage()
        notify("\nExample 1: GMOtrack GMO_EU_2005.tab ",addTimeStamp=false)
        notify("\nExample 2: GMOtrack GMO_EU_2005.tab -m 4 -c 100 \n",addTimeStamp=false)
    else:
        main(sys.argv[1:])
    
